#!/usr/bin/env node

const fs = require('fs');
const path = require('path');

const _ = require('lodash');
const pluralize = require('pluralize');
const prettier = require('prettier');

const schemas = (function() {
	const result = {};
	const schemaPath = path.resolve(process.cwd(), process.argv[2]);
	const outputPath = process.argv[3];
	const libPath = path.dirname(process.argv[1]);
	const templatesPath = './src/tpl';
	const underscoreSymbol = /_/;
	const entityWord = /\\.*?entit(y|ies).*?/i;

	const getFiles = directory => walkSync(directory);
	const walkSync = d => (fs.statSync(d).isDirectory() ? [].concat.apply([], fs.readdirSync(d).map(f => walkSync(path.join(d, f)))) : d); // [1], [2]
	const getSchemas = files => files.map(getSchema);
	const getSchema = file => require(path.resolve(file));

	const getPath = (srcPath, title = '', outputPath) => {
		const wordForms = getWordforms(title);
		const mapping = {
			entities: wordForms.plural.lowerFirst,
			Entities: wordForms.plural.upperFirst,
			entity: wordForms.singular.lowerFirst,
			Entity: wordForms.singular.upperFirst,
			[path.join(templatesPath, 'react')]: path.join(outputPath, 'src'),
			[path.join(templatesPath, 'angular')]: path.join(outputPath, 'src'),
			[path.normalize(templatesPath)]: path.normalize(outputPath)
		};

		const mappignRegExp = new RegExp(
			Object.keys(mapping)
				.map(k => k.split(path.sep).join('\\\\'))
				.join('|'),
			'g'
		);
		return srcPath.replace(mappignRegExp, m => mapping[m]);
	};

	const getWordforms = title => {
		const plural = pluralize.plural(title);
		const singular = pluralize.singular(title);

		return {
			plural: {
				camelCase: _.camelCase(plural),
				kebabCase: _.kebabCase(plural),
				lowerFirst: _.lowerFirst(plural),
				snakeCase: _.snakeCase(plural),
				upperFirst: _.upperFirst(plural)
			},
			singular: {
				camelCase: _.camelCase(singular),
				kebabCase: _.kebabCase(singular),
				lowerFirst: _.lowerFirst(singular),
				snakeCase: _.snakeCase(singular),
				upperFirst: _.upperFirst(singular)
			}
		};
	};

	const createFoldersRecursive = folders =>
		folders.reduce((acc, curr) => {
			const currentPath = path.resolve(acc, curr);
			const currentPathExists = fs.existsSync(currentPath);

			if (!currentPathExists) fs.mkdirSync(currentPath);

			return currentPath;
		}, process.cwd());

	const schemaFiles = getFiles(schemaPath);
	const templateFiles = getFiles(path.resolve(libPath, templatesPath)).map(f => path.relative(libPath, f));
	const separateTemplates = templateFiles.filter(templateFile => !templateFile.match(underscoreSymbol) && templateFile.match(entityWord));
	const commonTemplates = templateFiles.filter(templateFile => !templateFile.match(underscoreSymbol) && !templateFile.match(entityWord));
	const schemas = getSchemas(schemaFiles);
	const titles = schemas.map(({ title }) => title);

	schemas.forEach(({ title, properties }) => separateTemplates.forEach(template => (result[getPath(template, title, outputPath)] = prettier.format(require(path.resolve(libPath, template))(getWordforms(title), properties)))));
	commonTemplates.forEach(template => (result[getPath(template, '', outputPath)] = prettier.format(require(path.resolve(libPath, template))(titles))));

	const files = Object.keys(result);
	const outFolders = path.normalize(process.argv[3]).split(path.sep);
	const filesFolders = _.uniq(files.map(path.dirname).sort()).map(filePath => filePath.split(path.sep));

	createFoldersRecursive(outFolders);
	filesFolders.map(createFoldersRecursive);

	files.map(file => {
		if (!fs.existsSync(file)) {
			fs.writeFileSync(file, result[file]);
			console.log('\x1b[7m', `${file}`, '\x1b[0m');
		} else {
			console.log(file);
		}
	});
})();

module.exports = schemas;

// Thanks to
// [1] https://gist.github.com/kethinov/6658166
// [2] https://stackoverflow.com/questions/10865025/merge-flatten-an-array-of-arrays-in-javascript
